@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2010 Nokia Corporation.
- * Copyright (C) 2012 Intel Corporation.
+ * Copyright (C) 2012, 2013 Intel Corporation.
  *
  * Authors: Arun Raghavan <arun.raghavan@collabora.co.uk>
  *          Krzesimir Nowak <krnowak@openismus.com>
@@ -24,7 +24,6 @@
 #include "gupnp-dlna-profile-guesser.h"
 #include "gupnp-dlna-profile-guesser-impl.h"
 #include "gupnp-dlna-profile-loader.h"
-#include "gupnp-dlna-profile-backend.h"
 #include "gupnp-dlna-metadata-extractor.h"
 #include "gupnp-dlna-metadata-backend.h"
 
@@ -163,9 +162,9 @@ gupnp_dlna_profile_guesser_class_init
         /**
          * GUPnPDLNAProfileGuesser::done:
          * @profile_guesser: The #GUPnPDLNAProfileGuesser.
-         * @uri: URI for which profile was guessed.
-         * @dlna: The results as #GUPnPDLNAProfile.
-         * @error: Contains details of the error if discovery failed,
+         * @info: (transfer none): URI metadata as #GUPnPDLNAInformation.
+         * @dlna: (allow-none) (transfer none): The results as #GUPnPDLNAProfile.
+         * @error: (allow-none): Contains details of the error if discovery failed,
          * else is %NULL.
          *
          * Will be emitted when guessing DLNA profile for a URI has finished.
@@ -180,7 +179,7 @@ gupnp_dlna_profile_guesser_class_init
                               g_cclosure_marshal_generic,
                               G_TYPE_NONE,
                               3,
-                              G_TYPE_STRING,
+                              GUPNP_TYPE_DLNA_INFORMATION,
                               GUPNP_TYPE_DLNA_PROFILE,
                               G_TYPE_ERROR);
 
@@ -190,8 +189,9 @@ gupnp_dlna_profile_guesser_class_init
                 gboolean extended = ((iter) % 2 != 0); /* F,T,F,T */
                 guint rel_index = (relaxed ? 1 : 0);
                 guint ext_index = (extended ? 1 : 0);
-                GUPnPDLNAProfileLoader *loader =
-                      gupnp_dlna_profile_loader_get_default (relaxed, extended);
+                GUPnPDLNAProfileLoader *loader = gupnp_dlna_profile_loader_new
+                                        (relaxed,
+                                         extended);
 
                 profiles_list[rel_index][ext_index] =
                                gupnp_dlna_profile_loader_get_from_disk (loader);
@@ -246,19 +246,6 @@ unref_extractor_in_idle (GUPnPDLNAMetadataExtractor *extractor)
         return FALSE;
 }
 
-static GUPnPDLNAProfile *
-guess_profile (GUPnPDLNAProfileGuesser *guesser,
-               GUPnPDLNAInformation    *info)
-{
-        GUPnPDLNAProfileGuesserImpl *guesser_impl =
-                                 gupnp_dlna_profile_backend_get_guesser_impl ();
-        GList *profiles = gupnp_dlna_profile_guesser_list_profiles (guesser);
-
-        return gupnp_dlna_profile_guesser_impl_guess_profile (guesser_impl,
-                                                              info,
-                                                              profiles);
-}
-
 static void
 gupnp_dlna_discovered_cb (GUPnPDLNAProfileGuesser *guesser,
                           GUPnPDLNAInformation    *info,
@@ -268,14 +255,13 @@ gupnp_dlna_discovered_cb (GUPnPDLNAProfileGuesser *guesser,
         GUPnPDLNAProfile *profile = NULL;
         GUPnPDLNAMetadataExtractor *extractor =
                                       GUPNP_DLNA_METADATA_EXTRACTOR (user_data);
-        const gchar *uri = NULL;
 
         if (!error) {
-                profile = guess_profile (guesser, info);
+                profile = gupnp_dlna_profile_guesser_guess_profile_from_info
+                                        (guesser,
+                                         info);
         }
-        if (info)
-                uri = gupnp_dlna_information_get_uri (info);
-        g_signal_emit (guesser, signals[DONE], 0, uri, profile, error);
+        g_signal_emit (guesser, signals[DONE], 0, info, profile, error);
 
         g_idle_add ((GSourceFunc) unref_extractor_in_idle, extractor);
 }
@@ -299,16 +285,21 @@ gupnp_dlna_profile_guesser_guess_profile_async
                                         guint                     timeout_in_ms,
                                         GError                  **error)
 {
-        GUPnPDLNAMetadataExtractor *extractor =
-                                   gupnp_dlna_metadata_backend_get_extractor ();
+        GUPnPDLNAMetadataExtractor *extractor;
         gboolean queued;
-        GError *extractor_error = NULL;
-        guint id =
-                g_signal_connect_swapped (extractor,
-                                          "done",
-                                          G_CALLBACK (gupnp_dlna_discovered_cb),
-                                          guesser);
+        GError *extractor_error;
+        guint id;
 
+        g_return_val_if_fail (GUPNP_IS_DLNA_PROFILE_GUESSER (guesser), FALSE);
+        g_return_val_if_fail (uri != NULL, FALSE);
+        g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+        extractor = gupnp_dlna_metadata_backend_get_extractor ();
+        extractor_error = NULL;
+        id = g_signal_connect_swapped (extractor,
+                                       "done",
+                                       G_CALLBACK (gupnp_dlna_discovered_cb),
+                                       guesser);
         queued = gupnp_dlna_metadata_extractor_extract_async (extractor,
                                                               uri,
                                                               timeout_in_ms,
@@ -329,7 +320,9 @@ gupnp_dlna_profile_guesser_guess_profile_async
  * @guesser: #GUPnPDLNAProfileGuesser object to use for guessing.
  * @uri: URI of media.
  * @timeout_in_ms: Timeout of guessing in miliseconds.
- * @error: #GError object or %NULL.
+ * @dlna_info: (allow-none) (transfer full) (out): A place where to
+ * store DLNA information or %NULL.
+ * @error: (allow-none): #GError object or %NULL.
  *
  * Synchronously guesses DLNA profile for given @uri.
  *
@@ -340,32 +333,94 @@ gupnp_dlna_profile_guesser_guess_profile_sync
                                        (GUPnPDLNAProfileGuesser  *guesser,
                                         const gchar              *uri,
                                         guint                     timeout_in_ms,
+                                        GUPnPDLNAInformation    **dlna_info,
                                         GError                  **error)
 {
-        GError *extraction_error = NULL;
-        GUPnPDLNAMetadataExtractor *extractor =
-                                   gupnp_dlna_metadata_backend_get_extractor ();
-        GUPnPDLNAInformation *info =
-                 gupnp_dlna_metadata_extractor_extract_sync (extractor,
-                                                             uri,
-                                                             timeout_in_ms,
-                                                             &extraction_error);
-        GUPnPDLNAProfile *profile = NULL;
+        GError *extraction_error;
+        GUPnPDLNAMetadataExtractor *extractor;
+        GUPnPDLNAInformation *info;
+        GUPnPDLNAProfile *profile;
+
+        g_return_val_if_fail (GUPNP_IS_DLNA_PROFILE_GUESSER (guesser), NULL);
+        g_return_val_if_fail (uri != NULL, NULL);
+        g_return_val_if_fail (dlna_info == NULL || *dlna_info == NULL, NULL);
+        g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+        extraction_error = NULL;
+        extractor = gupnp_dlna_metadata_backend_get_extractor ();
+        info = gupnp_dlna_metadata_extractor_extract_sync (extractor,
+                                                           uri,
+                                                           timeout_in_ms,
+                                                           &extraction_error);
+        profile = NULL;
 
         if (extraction_error)
                 g_propagate_error (error,
                                    extraction_error);
         else
-                profile = guess_profile (guesser, info);
+                profile = gupnp_dlna_profile_guesser_guess_profile_from_info
+                                        (guesser,
+                                         info);
 
-        if (info)
-                g_object_unref (info);
+        if (info) {
+                if (dlna_info)
+                        *dlna_info = info;
+                else
+                        g_object_unref (info);
+        }
         if (extractor)
                 g_object_unref (extractor);
 
         return profile;
 }
 
+/**
+ * gupnp_dlna_profile_guesser_guess_profile_from_info:
+ * @guesser: The #GUPnPDLNAProfileGuesser object.
+ * @info: The #GUPnPDLNAInformation object.
+ *
+ * Guesses the profile which fits to passed @info.
+ *
+ * Returns: (transfer none): A #GUPnPDLNAProfile object on success,
+ * %NULL otherwise.
+ */
+GUPnPDLNAProfile *
+gupnp_dlna_profile_guesser_guess_profile_from_info
+                                        (GUPnPDLNAProfileGuesser *guesser,
+                                         GUPnPDLNAInformation    *info)
+{
+        GList *profiles;
+        GUPnPDLNAVideoInformation *video_info;
+        GUPnPDLNAAudioInformation *audio_info;
+        GUPnPDLNAImageInformation *image_info;
+        GUPnPDLNAProfile *profile;
+
+        g_return_val_if_fail (GUPNP_IS_DLNA_PROFILE_GUESSER (guesser), NULL);
+        g_return_val_if_fail (GUPNP_IS_DLNA_INFORMATION (info), NULL);
+
+        profiles = gupnp_dlna_profile_guesser_list_profiles (guesser);
+        video_info = gupnp_dlna_information_get_video_information (info);
+        audio_info = gupnp_dlna_information_get_audio_information (info);
+        image_info = gupnp_dlna_information_get_image_information (info);
+
+        if (image_info)
+                profile = gupnp_dlna_profile_guesser_impl_guess_image_profile
+                                        (info,
+                                         profiles);
+        else if (video_info)
+                profile = gupnp_dlna_profile_guesser_impl_guess_video_profile
+                                        (info,
+                                         profiles);
+        else if (audio_info)
+                profile = gupnp_dlna_profile_guesser_impl_guess_audio_profile
+                                        (info,
+                                         profiles);
+        else
+                profile = NULL;
+
+        return profile;
+}
+
 /**
  * gupnp_dlna_profile_guesser_get_profile:
  * @guesser: The #GUPnPDLNAProfileGuesser object.
@@ -375,7 +430,7 @@ gupnp_dlna_profile_guesser_guess_profile_sync
  *
  * Returns: (transfer none): A #GUPnPDLNAProfile object on success,
  * %NULL otherwise.
- **/
+ */
 GUPnPDLNAProfile *
 gupnp_dlna_profile_guesser_get_profile (GUPnPDLNAProfileGuesser *guesser,
                                         const gchar             *name)
@@ -385,6 +440,7 @@ gupnp_dlna_profile_guesser_get_profile (GUPnPDLNAProfileGuesser *guesser,
         GUPnPDLNAProfileGuesserPrivate *priv;
 
         g_return_val_if_fail (GUPNP_IS_DLNA_PROFILE_GUESSER (guesser), NULL);
+        g_return_val_if_fail (name != NULL, NULL);
 
         priv = guesser->priv;
 
@@ -409,7 +465,7 @@ gupnp_dlna_profile_guesser_get_profile (GUPnPDLNAProfileGuesser *guesser,
  *
  * Returns: (transfer none) (element-type GUPnPDLNAProfile): A #GList
  * of #GUPnPDLNAProfile on success, %NULL otherwise.
- **/
+ */
 GList *
 gupnp_dlna_profile_guesser_list_profiles (GUPnPDLNAProfileGuesser *guesser)
 {
